package BK::Composer;

=head1 NAME

generate - Generate a message from a hash and array reference

generate - Generate a hash and array reference from a fd (opened for reading)

=head1 SYNOPSIS

     use BK::Composer;

     $msg     = generate(\%hash, \@args);
     ($hash, $args) = ungenerate(FD);

=head1 DESCRIPTION

These routines allow you to read and write messages across
a BK Client/Server connection.

=cut

require 5.002;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(generate ungenerate);

sub generate {
    my ($table, $args) = @_;
    my ($res, $for) = ('', '');
    
    # print "composing for: $table->{func}\n";
    foreach $val (@$args)  {
        $for = "cNa" . (length($val) +1);
        $res .= pack($for, 0, length($val), $val);
    }

    # print "\nGenerating message with:\n";
    foreach $key (keys %$table) {
        $for = "a" . (length($key) +1) . "Na" . (length($table->{$key})+1);
        $res .= pack($for, $key, length($table->{$key}), $table->{$key});
    }
    return pack("CN", 19, length($res)) . $res;
}

sub ungenerate {
    my $fd = shift;
    my ($data, %table, @args) = ("", (), ());

    read($fd, $data, 5) == 5 || die 'Unable to read (Connection lost?).';
    my ($chk, $tot) = unpack("CN", $data);
    # print "header: chk = $chk\ttot = $tot\t";

    die 'You don\'t know the lingo. Bye.' if $chk != 19;

    read($fd, $data, $tot) == $tot || die 'Unable to read (Connection lost?).';
    while (length($data) >0) {
        my ($key, $len) = ("", 0);

        while ($len < length($data)) {
	    last if not unpack("x${len}c", $data);
            $len++;
        }
        if ($len) {
            ($key, $len) = unpack("a${len}xN", $data);
            $data = substr($data, length($key) +5);
            $val = unpack("a$len", $data);
            $data = substr($data, length($val) + 1);

            $table{$key} = $val;
        } else {
            $len = unpack("xN", $data);
            $data = substr($data, 5);
            $val = unpack("a$len", $data);
            $data = substr($data, length($val) + 1);

            push(@args, $val);
        }
    }
    # print "func = ", $table{func}, "\n";
    \(%table, @args);
}

1;
